#include "debug.h"
#include "loop_exp.h"

#define PIN_2_MASK 0x04
#define H13 digitalWrite(13, HIGH);
#define L13 digitalWrite(13, LOW);

#define FREQ_4MHz 4
#define FREQ_2MHz 2

#define CMD_OK 1
#define CMD_ERROR 64
#define CMD_READ 65
#define CMD_SETTINGS 66
#define CMD_STATUS 67

unsigned char data[1024];
unsigned int timeout;
unsigned char frequency = FREQ_2MHz;
unsigned char trigger = HIGH;

void readData(void);
void maskData(void);
void encodeData(void);

void setup() {
  Serial.begin(115200);
  pinMode(2, INPUT);
  pinMode(13, OUTPUT);
  L13;
  
  Serial.write(CMD_OK);
  delay(100);
}

void loop() {
  if (Serial.available()) { 
    switch(Serial.read()) {
      case CMD_READ:
        cli();
        readData();
        sei();
        maskData();
        encodeData();
        Serial.write(data, 128);
        Serial.flush();
        break;
      case CMD_SETTINGS:
        while (Serial.available() < 2) {delay(1);}
        frequency = Serial.read();
        trigger = Serial.read();
        Serial.write(CMD_OK);
        break;
      case CMD_STATUS:
        sprintf((char *)data,
                "Freq: %iMhz\nTrigger: %s\n",
                frequency,
                trigger ? "High" : "Low");
        Serial.print((char *)data);
        Serial.write(CMD_OK);
        Serial.flush();
        break;
      case 't':
        readData();
        maskData();
        for (int i = 0; i < 1024; i++)
          Serial.print((int)data[i]);
        Serial.println();
        break;
    }
  }
}

void readData() {
  unsigned char *data_ptr = data;
  unsigned int samplesLeft = 1024;
  unsigned char tmp = 0;
  timeout = 0xffff;
  
  if (trigger == HIGH) {
    while (PIND & 0x04)
      if (!--timeout) return;
    timeout = 0xffff;
    while (!(PIND & 0x04))
      if (!--timeout) return;
  } else {
        while (!(PIND & 0x04))
      if (!--timeout) return;
    timeout = 0xffff;
    while (PIND & 0x04)
      if (!--timeout) return;
  }
  
  if (frequency == FREQ_4MHz) {
    READ_4MHz;
  } else {
    asm volatile (
    "ldi  r25, 0x04   \n"
    "ldi  r24, 0x00   \n"
    "read_loop:       \n"
    "IN   r21, %1     \n"
    "ANDI r21, 0x04   \n"
    "ST  %a0+, r21    \n"
    "SBIW r24, 0x01   \n"
    "BRNE read_loop   \n"
    :"+e" (data_ptr)
    :"I"  (_SFR_IO_ADDR(PIND))
    :"r21", "r25", "r24"
    );
  }
}

void maskData() {
  if (frequency == FREQ_2MHz)
    return; // Already computed with extra time left over during sampling
  
  for (int i = 0; i < 1024; i++) {
    data[i] = data[i] & 0x04;
  }
}

void encodeData() {
  unsigned char b;
  for (int i = 0; i < 1024; i++) {
    if (data[i] != 0)
      data[i / 8] |= 1 << (7 - (i % 8));
    else
      data[i / 8] &= ~(1 << (7 - (i % 8)));
  }
}
